import java.sql.*;
import java.util.ArrayList;

/**
 * Class that contains methods for accessing an SQL database
 * @author ilaggoodly
 *
 */
public class Database{
	private String tableName;
	private Connection con;
	
	/**
	 * Deletes a vip from the database
	 * @param vip --a vip to delete
	 * @throws SQLException --they didn't exist in the first place
	 */
	public void delVIP(VIP vip) {
		try{
			Statement del = con.createStatement();
			del.executeUpdate("DELETE FROM " +tableName +" WHERE bnetName =('"+vip.getName()+"')" );
		}catch(SQLException e1){
			System.out.println("No user to delete");
		}
	}
	
	/**
	 * Creates the table for this database, returns true if successful, false otherwise
	 * @return --true if successful (i.e. no prior table), false otherwise
	 */
	public boolean makeTable(){
		try{
			Statement createTable = con.createStatement();
			createTable.executeUpdate("CREATE TABLE "+ tableName+ " (" +
					"bnetName VARCHAR(16), " +
					"status VARCHAR(80), " +
					"rank VARCHAR(60), " +
					"banlistName VARCHAR(50), " +
					"lastOnline VARCHAR(40), " +
					"lastUpdate VARCHAR(40) NOT NULL, " +
					"online TINYINT NOT NULL, "+
					"PRIMARY KEY (bnetName))");
		}catch (SQLException e1){
			return false; 
		}
		return true;
	}
	
	/**
	 * Queries the database for a VIP given their name
	 * @param bnetName --the name of the VIP to look for
	 * @return --a VIP represented in the Database, or null if they don't exist
	 */
	public VIP getVIPbyName(String bnetName) {
		try{
			Statement getByName = con.createStatement();
			ResultSet rs = getByName.executeQuery("SELECT * FROM " + tableName + 
					" WHERE bnetName =('" + bnetName +"')");
			
			while(rs.next()){
				VIP theVIP = new VIP(bnetName,rs.getString("rank"),rs.getString("status"), 
						rs.getString("banlistName"), rs.getBoolean("online"));
				theVIP.setLastOnline(rs.getString("lastOnline"));
				return theVIP;
			}
			return null; //no1 found
		}catch(SQLException e1){
			return null; //error
		}
	}
	/**
	 * Adds a VIP to the Database
	 * @param vip --A VIP to add to the database (not there yet)
	 * @throws --SQLException if the user is already there
	 */
	public void addVIP (VIP vip){
		Statement stmt;
		try{
			stmt = con.createStatement();
			stmt.executeUpdate("INSERT INTO "+ tableName +" (bnetName, status ,rank , banlistName," +
					"lastOnline,lastUpdate,online) VALUES ('"+vip.getName()+"','"+vip.getStatus()+"','" 
					+ vip.getRank()+ "','" + vip.getBAccount() + "','" + vip.getLastOnline() + "','" +
					vip.getlastUpdate() + "'," + (vip.getOnline()? 1 : 0) +")");
		}catch(SQLException e1){
			updateRankBanlist(vip);
		}

	}
	/**
	 * Adds a mass amount of VIPs using a prepared statement... (unused)
	 * @param vips
	 */
	public void massAdd (ArrayList<VIP> vips){
		try{
			String sql = "INSERT INTO "+ tableName +" (bnetName, status ,rank , banlistName," +
			"lastOnline,lastUpdate,online) VALUES (?,?,?,?,?,?,?)";
			PreparedStatement add = con.prepareStatement(sql);
			
			for(VIP vip : vips){
				add.setString(1,vip.getName());
				add.setString(2, vip.getStatus());
				add.setString(3, vip.getRank());
				add.setString(4, vip.getBAccount());
				add.setString(5, vip.getLastOnline());
				add.setString(6, ""+vip.getlastUpdate());
				add.setBoolean(7, vip.getOnline());
				
				add.executeUpdate();
			}
		}catch(SQLException e1){
			e1.printStackTrace();
		}
		
	}
	
	/**
	 * Updates a VIP's rank and banlist name
	 * @param vip --A VIP to update
	 */
	private void updateRankBanlist(VIP vip){
		Statement update;
		try{
			update = con.createStatement();
			update.executeUpdate("UPDATE " + tableName + " SET " +
					"rank = '" + vip.getRank() +
					"', banlistName = '" + vip.getBAccount()+ "'"
					+ " WHERE bnetName = '"+vip.getName()+"'");
		}catch(SQLException e1){
			System.out.println("error updating "+vip.getName());
			e1.printStackTrace();
		}
	}
	/**
	 * Updates the status of a VIP (and the lastOnline, lastUpdate times as well)
	 * @param vip --the VIP to update
	 */
	public void updateStatus(VIP vip){
		try{
			Statement update;
			update = con.createStatement();
			update.executeUpdate("UPDATE " + tableName + " SET "+ 
					"status = '" + vip.getStatus() +"', " +
					"lastOnline = '" +vip.getLastOnline() + "', "+
					"lastUpdate = '" + vip.getlastUpdate()+ "', " +
					"online = '" + (vip.getOnline() ? 1 : 0)+"' " +
							"WHERE bnetName = '" + vip.getName()+"'");
		}catch(SQLException e1){
			e1.printStackTrace();
			System.out.println("error updating status " +vip.getName() );
		}
	}
	
	/**
	 * Creates an instance of Database and attempts to load the Connector/J driver
	 * @param url --the url of the mysql db
	 * @param user --the name of the user in the mysqldb (with execute + select access)
	 * @param pw --user's password
	 * @param dbName --a unique name for the table of data
	 */
	public Database (String url, String user, String pw, String dbName){
		try{ //attempt to load Connector/J
			Class.forName("com.mysql.jdbc.Driver");
		}catch (ClassNotFoundException e1){
			System.out.println("Couldn't load Connector/J");
		}
		try{
			con = DriverManager.getConnection(url, user, pw);
		}catch (SQLException e2){
			System.out.println("Couldn't connect to mysqldb");
		}
		tableName = dbName;
	}
}