import java.io.DataInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Properties;
import java.util.Scanner;
import java.util.TimerTask;
import java.util.ArrayList;

import javax.swing.JCheckBox;
import javax.swing.JComponent;

import callback_interfaces.PluginCallbackRegister;
import callback_interfaces.PublicExposedFunctions;
import callback_interfaces.StaticExposedFunctions;
import exceptions.PluginException;
import plugin_interfaces.EventCallback;
import plugin_interfaces.GenericPluginInterface;
import plugin_interfaces.OutgoingTextCallback;


/**
 * 3/5/2009
 * A plugin for /whoising people on a wc3banlist style viplist and creating an html page from
 * the info. Should not be used in conjunction with any other plugins, esp. Anti-flood
 * as it will desync the /whoising from the parsing.
 * 
 * @author ilaggoodly
 * 
 *
 */
public class PluginMain extends GenericPluginInterface implements OutgoingTextCallback, EventCallback
{
    private PublicExposedFunctions out; 		//how the plugin communicates with the bot
    public ArrayList<VIP> VIPlist = new ArrayList<VIP>(); //the main backend for users in the VIP list
    private boolean logging = true; 			//Whether or not to print out what the bot is doing
    private int checkIndex = 0; 				//the last person in VIPlist that got checked
    private boolean onStartup = true; 			//indicates whether we should call /whois or do nothing...
    Callback doWhois; 							//the timertask that does the /whois checking
    private int delay = 0;						//don't edit.
    private Database DB;
    
    public final String unchecked ="Not checked yet!";
    public final String errcheck ="Error Checking";
    public final String offline ="Not Online";
    
    public void load(StaticExposedFunctions staticFuncs)
    {
    }
   /**
    * called when the plugin is registered, loads some default values, and instantiates the VIPlist
    */
    public void activate(PublicExposedFunctions out, PluginCallbackRegister register)
    {
        this.out = out;
        doWhois = null;
        register.registerOutgoingTextPlugin(this, null);
        register.registerEventPlugin(this, null);

        try{
        	String mysqlURL  = "jdbc:mysql://localhost/"+out.getLocalSetting(getName(),"Mysql Database Name");
        	String mysqlUser = out.getLocalSetting(getName(),"Mysql Database User");
        	String mysqlPW = out.getLocalSetting(getName(),"User Password");
        	DB = new Database(mysqlURL, mysqlUser, mysqlPW, out.getName());
        	if(DB.makeTable())
        		System.out.println("making new table");
        	else
        		System.out.println("table already made");
        	
        }catch (Exception ex){
        	System.out.println("Error with database configuration");
        	ex.printStackTrace();
        }
        
        logging = Boolean.parseBoolean(out.getLocalSettingDefault(getName(), "Debug", "true"));
        parseVIPList(out.getLocalSettingDefault(getName(), "VIP list", "http://banlist.us/vip.php"),new ArrayList<VIP>());
        onStartup = Boolean.parseBoolean(out.getLocalSettingDefault(getName(), "Start On Connect", "false"));
        
        if(logging)
        	System.out.println("reading from" + out.getLocalSettingDefault(getName(), "VIP list", "http://banlist.us/vip.php"));
        if(VIPlist!=null&&onStartup){//make sure this worked out
        	doWhois = new Callback();
        	out.schedule(doWhois, 1000 * Integer.parseInt(out.getLocalSettingDefault(getName(), "Delay", "10")));
        }
        
    }

    public void deactivate(PluginCallbackRegister register){
    	
    }

    public String getName(){
        return "VIPWatch";
    }

    public String getVersion(){
        return "2.2";
    }

    public String getAuthorName(){
        return "ilaggoodly";
    }

    public String getAuthorWebsite(){
        return "www.banlist.nl/phpBB3";
    }

    public String getAuthorEmail(){
        return "girlshalo@yahoo.com";
    }

    public String getLongDescription(){
        return "Periodically checks an online *or local* VIP list (wc3banlist format) to see if the people are online, and generates an html page with their status!";
    }

    public Properties getDefaultSettingValues()
    {
        Properties p = new Properties();
        p.setProperty("VIP list", "http://banlist.us/vip.php");
        p.setProperty("Output file", System.getProperty("user.dir")+"/whosonline.html");
        p.setProperty("Delay", "10");
        p.setProperty("Debug","false");
        p.setProperty("Start On Connect","false");
        p.setProperty("Mysql Database Name", "vipwatch");
        p.setProperty("Mysql Database user", "javaop2");
        p.setProperty("User Password", "");
        
        return p;
    }

    public Properties getSettingsDescription()
    {
    	Properties p = new Properties();
        p.setProperty("VIP list", "This is the location of the VIP LIST to parse and check for online-ness, can be http:// or a local URI");
        p.setProperty("Output file", "This is the location that this plugin will output the html file to, you should enter where you want it on your webserver");
        p.setProperty("Delay", "How long the bot will pause in between /whois checks, you can make it longer if your bot is getting disconnected");
        p.setProperty("Debug", "Display error messages in console or not, (can get a bit messy)");
        p.setProperty("Start On Connect","Sets whether the bot will start with /whois after it connects, or you have to start with \"?start\"");
        p.setProperty("Mysql Database Name", "The name of the database you want to store the information in");
        p.setProperty("Mysql Database user", "A user with execute + select + create access in the above database");
        p.setProperty("User Password", "That user's password");
        
        return p;
    }
    /**
     * Creates a checkbox for the "Debug" and "Start on Connect" Properties
     */
    public JComponent getComponent(String settingName, String value)
    {
    	if(settingName.equalsIgnoreCase("Debug"))
            return new JCheckBox("", value.equalsIgnoreCase("true"));
        if(settingName.equalsIgnoreCase("Start On Connect"))
        	return new JCheckBox("", value.equalsIgnoreCase("true"));
        	
        return null;
    }

    //no global settings, they are all bot-specific
    public Properties getGlobalDefaultSettingValues()
    {
        Properties p = new Properties();
        return p;
    }

    public Properties getGlobalSettingsDescription()
    {

        Properties p = new Properties();
        return p;
    }

    public JComponent getGlobalComponent(String settingName, String value)
    {
        return null;
    }

    /**
     * A task scheduled to the bot to periodically /whois the next person in the VIPlist
     * It also attempts to check people who are Online more often
     * @author ilaggoodly
     *
     */
    private class Callback extends TimerTask
    {
    	public boolean pause = false;
    	
        public void run()
        {	
        	if(pause) //so we can pause it to prevent from being kicked...
        		return;
        	try
        	{
        		VIP theVIP;
        		
        		if (checkIndex>=VIPlist.size()) //reloop!
        			checkIndex = 0;
        		theVIP = VIPlist.get(checkIndex);
        		checkIndex++; //bump the index
        		
        		out.sendText("/whois "+theVIP.getName());
        		delay = 0;

        	}
        	catch(IOException e)
        	{
        		System.out.println(e);
        	}
        }
    }

    public String queuingText(String text, Object data)
    {
        return text;
    }

    public void queuedText(String text, Object data)
    {
    }

    public String nextInLine(String text, Object data)
    {
        return text;
    }

    public long getDelay(String text, Object data)
    {
        return delay;
    }

    public boolean sendingText(String text, Object data)
    {
        return true;
    }

    public void sentText(String text, Object data)
    {
    	
    }
    public void talk(String user, String statstring, int ping, int flags) throws IOException, PluginException
    {
    	doTrigger(user, statstring);
    }

    public void whisperFrom(String user, String statstring, int ping, int flags) throws IOException, PluginException
    {
    	doTrigger(user, statstring);
    }
    
    /**
     * Parse the messages from Bnet as whois, (and update the status accordingly), Someone was online!
     */
    public void info(String user, String statstring, int ping, int flags) throws IOException, PluginException
    {   
    	if(statstring.contains("Throne")){
    		VIP tmpVIP;
    		if (checkIndex>0)
    			tmpVIP = VIPlist.get(checkIndex-1);
    		else
    			tmpVIP = VIPlist.get(VIPlist.size()-1);
    		
    		tmpVIP.setStatus(statstring.split("Throne")[1],true);
    		DB.updateStatus(tmpVIP);
    	}
    }
    
    /**
     * checking someone1 not logged on results in an error (i.e. not online)
     */
    public void error(String user, String statstring, int ping, int flags) throws IOException, PluginException
    {

    	VIP tmpVIP;
    	if (checkIndex>0)
    		tmpVIP = VIPlist.get(checkIndex-1);
    	else
    		tmpVIP = VIPlist.get(VIPlist.size()-1);

    	tmpVIP.setStatus("Offline" , false);
    	DB.updateStatus(tmpVIP);

    		
    }

    
    /**
     * Performs the command passed to it by a user with tags (ANL)
     * @param user --name of the user calling the command
     * @param command --the text of the command
     */
    public void doTrigger(String user, String command){
    	
    	if(out.dbHasAny(user, "A", true)){ //only admins can do this stuff
    		
    		if(command.trim().equalsIgnoreCase("?reloadvip")){ //reload the VIP list
    			if(logging)
    				System.out.println("reloading vip list...");
    			String list = out.getLocalSettingDefault(getName(), "VIP list", "http://banlist.us/vip.php");
    			parseVIPList(list,VIPlist);
    			respondToCommand(user, "Reloaded VIP list " + list);
    		}
    		
    		else if(command.trim().equalsIgnoreCase("?start")){ //start the /whois
    			if(doWhois==null){
    				if(logging)
    					System.out.println("Starting the /whois check");
    				doWhois = new Callback();
    				out.schedule(doWhois, 1000 * Integer.parseInt(out.getLocalSettingDefault(getName(), "Delay", "5")));
    				respondToCommand(user,"Starting the /whois check");
    			}else{
    				respondToCommand(user,"Alreading running /whois check");
    			}
    			
    			
    		}else if(command.trim().equalsIgnoreCase("?stop")){ //stop the /whois
    			if(doWhois!=null){
    				respondToCommand(user, "Stopped the /whois check");
    				out.unschedule(doWhois);
    				doWhois =null;
    			}else{
    				respondToCommand(user, "Not running the /whois check");
    			}
    		}
    	}else{
    		//no reason to care... and lets not try to get ipbanned here...
    	}
    	
    }
    /**
     * Responds to a command, trying to avoid getting flooded
     * @param user
     * @param response
     */
    public void respondToCommand(String user, String response){
    	try{
    		doWhois.pause = true;
    		delay =5;
			out.sendText("/w " + user + " " + response);
			doWhois.pause = false;
		}catch (IOException e2){
			//do nothing
		}catch (NullPointerException e3){
			try{
				out.sendText("/w " + user + " " + response);
			}catch(Exception e4){
				//do nothing
			}
		}
    }

    /**
     * parses a VIP list into the arraylist of VIPs given a string
     * @param URL --teh location of the file to parse
     * @param VIPs --the list of VIPs to put the text file into
     * @return --true if successful, false otherwise
     */
    private void parseVIPList(final String url, final ArrayList<VIP> oldvips){

    	Scanner scan = null;
    	ArrayList<VIP> VIPs = new ArrayList<VIP>();

    	//check to see if it is a local file, or an http file
    	if(!url.toLowerCase().startsWith("http")){
    		try{
    			scan = new Scanner(new File(url));
    		}catch(FileNotFoundException e){
    			System.out.println("Err resolving local filename");
    			return;
    		}
    	}else{
    		try{
    			URLConnection urlConn; 
    			urlConn = new URL(url).openConnection(); 
    			urlConn.setDoInput(true); 
    			urlConn.setUseCaches(false); 
    			scan = new Scanner(new DataInputStream(urlConn.getInputStream()));

    		}catch (Exception e1){
    			System.out.println("Bad URL Syntax");
    			e1.printStackTrace();
    			return;
    		}
    	}

    	while(scan.hasNextLine()){
    		String cur_line = scan.nextLine();
    		String [] pieces = cur_line.split(" ",2); //username doesn't have a space
    		String vipName = pieces[0];

    		pieces = pieces[1].split(",",2); //Rank seperated by ,
    		String vipRank = pieces[0];

    		pieces = pieces[1].split(":",2);
    		String vipBAccount = pieces[1].trim();
    		VIP thisVIP = new VIP(vipName, vipRank, unchecked, vipBAccount ,false);
    		VIPs.add(thisVIP);
    		DB.addVIP(thisVIP); 
    		//adds the VIP to the database, or if they are already there,
    		//it updates their rank and banlistname
    	}
    	scan.close();
    	scan = null;

    	if (oldvips.size()>VIPs.size()){ //get rid of the VIPS that were removed...
    		for (VIP vip : oldvips){
    			if(VIPs.indexOf(vip)<0){
    				DB.delVIP(vip);
    			}
    		}
    	}
    	VIPlist =VIPs;
    
	}
    	
    
    //==methods required to implement an interface, but unused=======
    
    public void emote(String user, String statstring, int ping, int flags) throws IOException, PluginException
    {
    }


    public void whisperTo(String user, String statstring, int ping, int flags) throws IOException, PluginException
    {
    }

    public void userShow(String user, String statstring, int ping, int flags) throws IOException, PluginException
    {
    }

    public void userJoin(String user, String statstring, int ping, int flags) throws IOException, PluginException
    {
    }

    public void userLeave(String user, String statstring, int ping, int flags) throws IOException, PluginException
    {
    }

    public void userFlags(String user, String statstring, int ping, int flags) throws IOException, PluginException
    {
    }
    
    public void broadcast(String user, String statstring, int ping, int flags) throws IOException, PluginException
    {
    }

    public void channel(String user, String statstring, int ping, int flags) throws IOException, PluginException
    {
    }
}



