import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;


public  class VIP{
		private String name;
		private String rank;
		private String status;
		private String baccount;
		private long lastUpdate;
		private Date lastOnline;
		private boolean isOnline;
		
		private final String offline = "Offline";
		private final String errcheck = "Error Checking Status";
		private final String dateFormat = "yyyy.MM.dd 'at' HH:mm:ss";
		
		/**
		 * A constructor for a VIP
		 * @param Name --the bnet name of this vip
		 * @param Rank --the Rank of this VIP
		 * @param Status --the status of this VIP (online, offline,etc.)
		 * @param BAccount --The Banlist account of this VIP
		 */
		public VIP(String Name, String Rank, String Status,String BAccount,boolean isonline){
    		name = Name;
    		rank = Rank;
    		status = Status;
    		baccount = BAccount;
    		lastUpdate = System.currentTimeMillis();
    		lastOnline = null;
    		isOnline = isonline;
		}
		
    	public String getName(){
    		return name;
    	}
    	public String getRank(){
    		return rank;
    	}
    	public String getStatus(){
    		return status;
    	}
    	public String getBAccount(){
    		return baccount;
    	}
    	
    	/**
    	 * Sets the user's status, and updates the last time checked, and last Online
    	 * also auto-capitalizes the message
    	 * @param Status --a Bnet Status message to update to
    	 * @param online --if they are online or not
    	 */
    	public void setStatus(String Status, boolean online){
    		Status = Status.trim();
    		if(!Status.equalsIgnoreCase(offline)&&!Status.equalsIgnoreCase(errcheck))
    			lastOnline = new Date();
    		lastUpdate = System.currentTimeMillis();
    		status = Status.toUpperCase().charAt(0) + Status.substring(1);
    		isOnline = online;
    	}
    	
    	/**
    	 * gets the last time this VIP's status was updated
    	 * Will start with some weird values, but that doesn't matter
    	 * @return --a string representing the last time they were online
    	 * 
    	 * UNUSED
    	 */
    	public long getlastUpdate(){
	    	return lastUpdate/1000;
    	}
    	
    	/**
    	 * Gets the last time this person was online
    	 * @return --Last time they were online or "Unknown" if they've never been seen
    	 */
    	public String getLastOnline(){
    		try{
    			SimpleDateFormat dF = new SimpleDateFormat(dateFormat);
    			return dF.format(lastOnline);
    		}catch (NullPointerException e1){
    			return "Unknown";
    		}
    	}
    	
    	/**
    	 * Sets the last Online Date given a String with the proper format
    	 * @param formattedDate
    	 * @return --false if not completely parsed, true otherwise
    	 */
    	public boolean setLastOnline(String formattedDate){
    		ParsePosition pp= new ParsePosition(0);
    		lastOnline = (new SimpleDateFormat(dateFormat))
    			.parse(formattedDate,pp);
    		if(pp.getIndex() == formattedDate.length()-1)
    			return true;
    		else return false;
    	}
    	
    	/**
    	 * a toString() method
    	 * @Override --toString used for debugging and writing to rawOut.txt
    	 */
    	public String toString(){
    		return name + "::::" + rank + "::::" + status + "::::" + baccount+
    			"::::"+getlastUpdate()+ "::::" + getLastOnline();
    	}
    	/**
    	 * Overrides Object Equals to compare names only
    	 * @Override --Object equals
    	 */
    	public boolean equals (Object other){
    		if(name.equalsIgnoreCase(((VIP)other ).getName()))
    			return true;
    		else return false;
    	}
    	/**
    	 * gets if they are online or not
    	 * @return --true if the VIP is online, false otherwise.
    	 */
    	public boolean getOnline(){
    		return isOnline;
    	}
    }
